#!/usr/bin/perl -w
#------------------------------------------------------------------------------
# PROGRAM:      mudlist_client.pl
# DESCRIPTION:  Perl script for generating a mud list HTML file and INFO file
#               for AberMUDs from the mudlist_client.csv file, which is 
#               generated by the server program mudlist_server.pl.
#               This script is Copyright (C) 2004 by Alf Nathan (Ellyll). You
#               may use and modify this script for your own MUD, so long as the
#               MUD is TOTALLY free, i.e. no charge what so ever.
# VERSION:      V0.3 (2004-04-24)
#
# Modification History:
#
# Date         Version  Author                  Comments
# 2001-08-04   0.1      A. Nathan.              Created original version.
# 2002-01-28   0.2      A. Nathan.              Added config file code and
#                                               for using GMT time and having
#                                               the last updated time passed
#                                               using a dummy mud. Also uses
#                                               hostname headers for HTTP.
# 2004-04-23   0.3      A. Nathan               Changed to produce XHTML 1.0
#                                               Transitional compliant web
#                                               page.
#
#------------------------------------------------------------------------------

use strict;
use IO::Socket;
use Time::Local;

my $line;      # the line read from the mudlist data file
my $linenumber;# current line number

my $mudid;     # the unique ID for each mud as specified in the mudlist data file
my $mudname;   # the name of the mud
my $mudurl;    # the URL of the mud's website
my $mudaddr;   # the address of the mud (host name)
my $mudcurrip; # the current numeric IP address of the mud (if known)
my $mudlastip; # the last numeric IP address of the mud
my $mudlastconn; # the last numeric IP address of the mud
my $mudport;   # the mud's port
my $mudcomment;# comment about mud
my $mudstatus; # status of the mud

my $mudinfo;   # just a line to generate the output of record for the html and info file

my %muds;      # Hash of the muds

my $mudcount;  # Just a counter
my $mudsup;    # Number of muds that are up
my $mudsdown;  # Number of muds that are down
my $mudstotal; # Total number of muds

my $currtime;  # Current date/date
my $lastupdate;# Time mudlist was last updated (in local time)

my $sock;      # socket for connecting to the mud
my $str;       # Just used for a temporary string

my $finished_headers; # Set to 1 if finished processing HTTP headers

# Config file variables
my $CONFIG_FILE="mudlist_client_config.txt"; # location of the config file
my $paramid;            # Config parameter, e.g. HTML_FILE
my $paramvalue;         # Value of parameter
my %mudlist_config;     # Hash for the config info
my $configcount;        # Number of actual config lines
my @mudlist_params = ( 'HTML_FILE',
                       'HTML_FILE_URL',
                       'INFO_FILE',
                       'TMP_HTML_FILE',
                       'TMP_INFO_FILE',
                       'STYLESHEET_URL',
                       'CLIENT_FILE',
                       'INFO_COL_TITLE',
                       'INFO_COL_TABLE_HEADINGS',
                       'INFO_COL_TABLE_LINES',
                       'INFO_COL_LINE1',
                       'INFO_COL_LINE2',
                       'INFO_COL_UP',
                       'INFO_COL_DOWN',
                       'INFO_COL_MUDS_UP_TXT',
                       'INFO_COL_MUDS_UP_NUM',
                       'INFO_COL_MUDS_DOWN_TXT',
                       'INFO_COL_MUDS_DOWN_NUM',
                       'INFO_COL_TOTAL_MUDS_TXT',
                       'INFO_COL_TOTAL_MUDS_NUM',
                       'INFO_COL_UPDATED_TXT',
                       'INFO_COL_UPDATED_DTE',
                       'INFO_TXT_WEB_VER',
                       'INFO_COL_WEB_VER',
                       'INFO_COL_URL',
                       'INFO_COL_CHANGES',
                       'INFO_TXT_CHANGES',
                       'INFO_COL_SERVER_ADMIN',
                       'HTML_TXT_CHANGES',
                       'SERVER_ADDR',
                       'SERVER_PORT',
                       'SERVER_PATH',
                       'SERVER_ADMIN' );
my $mudlist_param;      # Config parameter (used in loop)
my $found;              # 0 or 1 if paramter found in loaded info

my $HTMLcol;            # Uder for alternating line colour in HTML file
my $INFOcol;            # Uder for alternating line colour in INFO file


# Read in the configuration info from the config file
open CONFIG_FILE, "<$CONFIG_FILE" or die "Can't open mud list client config file: $CONFIG_FILE error: $!\n";

# Read through the mudlist data file and load muds into a hash
$configcount = 0;
$linenumber = 0;
while (defined($line = <CONFIG_FILE>))
{
  chomp $line; # strip the \n from the end of the line
  $linenumber++;

  if ( substr($line,0,1) eq '#' or length($line) == 0 )
  {
    #print "Gotta blank line!\n";
    next;
  }

  # split the line up into the variables
  ($paramid,$paramvalue) = split('=', $line);

  $configcount++;

  #print "DEBUG: loading data for config parameter: $paramid = $paramvalue\n";
  #print "                       '$paramid',\n";

  $found = 0;
  foreach $mudlist_param (@mudlist_params)
  {
    if ( $mudlist_param eq $paramid )
    {
      $found = 1;
      last;
    }
  }
  if ( $found == 0 )
  {
    die "Error - invalid parameter '$paramid' specified in line $linenumber of config file $CONFIG_FILE\n";
  }

  $mudlist_config{$paramid} = $paramvalue;
}
# Finished reading in all the muds, close the file
close(CONFIG_FILE);

foreach $mudlist_param (@mudlist_params)
{
  if ( !defined($mudlist_config{$mudlist_param}) )
  {
    die "Error - paramater $mudlist_param not defined in config file $CONFIG_FILE\n";
  }
  if ( length($mudlist_config{$mudlist_param}) == 0 )
  {
    die "Error - no value specified for paramater $mudlist_param in config file $CONFIG_FILE\n";
  }
}


#print "Number of lines in config file:  $linenumber\n";
#print "Number of parameters configured: $configcount\n";

# The config data has been loaded, now on to the real business :)


# Lets try and download a new client data file first
$sock = new IO::Socket::INET (
                               PeerAddr => $mudlist_config{'SERVER_ADDR'},
                               PeerPort => $mudlist_config{'SERVER_PORT'},
                               Proto => 'tcp',
                             );
if ( !$sock )
{
  die "Can't open socket for downloading client mudlist data from server! $!\n";
}

#print $sock "GET $mudlist_config{'SERVER_PATH'}\n";
#print $sock "GET $SERVERPATH HTTP/1.0\nHost: $SERVERADDR\n\n";
print $sock "GET $mudlist_config{'SERVER_PATH'} HTTP/1.0\nHost: $mudlist_config{'SERVER_ADDR'}\n\n";

open CLIENT_FILE, ">$mudlist_config{'CLIENT_FILE'}" or die "Can't open client data file $mudlist_config{'CLIENT_FILE'} for writing, error: $!\n";
print CLIENT_FILE <$sock>;
close(CLIENT_FILE);


# Open the mudlist client data file
open CLIENT_FILE, "<$mudlist_config{'CLIENT_FILE'}" or die "Can't open mud list client data file: $mudlist_config{'CLIENT_FILE'} error: $!\n";

# Open the HTML and INFO output files
open TMP_HTML_FILE, ">$mudlist_config{'TMP_HTML_FILE'}" or die "Can't open temporary HTML output file: $mudlist_config{'TMP_HTML_FILE'} error: $!\n";
open TMP_INFO_FILE, ">$mudlist_config{'TMP_INFO_FILE'}" or die "Can't open temporary INFO output file: $mudlist_config{'TMP_INFO_FILE'} error: $!\n";

# Print file headers
print TMP_HTML_FILE <<EOF;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html;    charset=utf-8" />
<title>AberMUD List</title>
<link rel="stylesheet" href="$mudlist_config{'STYLESHEET_URL'}" type="text/css" />
</head>
<body>
<center>
<h1 class="title">AberMUD List</h1>
<br />
<table cellpadding="10">
  <tr class="tableheading">
    <th>MUD Name</th><th>Address</th><th>IP Address</th><th>Port</th><th>Status</th><th>Comments</th>
  </tr>
EOF

print TMP_INFO_FILE <<EOF;
                                 $mudlist_config{'INFO_COL_TITLE'}AberMUD List&*

EOF
printf TMP_INFO_FILE "%s%-20s %-33s %-15s %-4s %s\n",
                   $mudlist_config{'INFO_COL_TABLE_HEADINGS'},
                   "        Name","              Host","  IP Address","Port"," Up";
printf TMP_INFO_FILE "%s%-20s %-33s %-15s %-4s %s\n",
                   $mudlist_config{'INFO_COL_TABLE_LINES'},
                   "--------------------",
                   "---------------------------------",
                   "---------------","----","----";

# Read through the mudlist data file and load muds into a hash
$mudcount = 0;
$finished_headers = 0;
while (defined($line = <CLIENT_FILE>))
{
  chomp $line; # strip the \n from the end of the line

  #$str = "DEBUG readling line (length: " . length($line) . "): $line\n";
  #print $str;

  # skip the HTTP headers, we dont care about them
  if ( $finished_headers == 0 )
  {
    if ( $line eq "\r" )
    {
      $finished_headers = 1;
    }
    next;
  }

  # split the line up into the variables
  ($mudid,$mudname,$mudurl,$mudaddr,$mudcurrip,$mudport,$mudlastconn,$mudstatus,$mudcomment) = split(',', $line);

  # If it's the dummy mud read the last up time and put that as the last update time
  if ( $mudid == 0 )
  {
    $lastupdate = localtime(timegm(split('\|',$mudlastconn)));
    next;
  }

  $mudcount++;

  #print "DEBUG: loading data for mud: $mudid:$mudname:$mudurl:$mudaddr:$mudport:$mudcomment\n";

  $muds{$mudid}{name}     = $mudname;
  $muds{$mudid}{url}      = $mudurl;
  $muds{$mudid}{addr}     = $mudaddr;
  $muds{$mudid}{currip}   = $mudcurrip;
  $muds{$mudid}{port}     = $mudport;
  $muds{$mudid}{lastconn} = $mudlastconn;
  $muds{$mudid}{status}   = $mudstatus;
  $muds{$mudid}{comment}  = $mudcomment;
}
# Finished reading in all the muds, close the file
close(CLIENT_FILE);
$mudstotal = $mudcount;


# Loop through all the muds in the hash
$mudsup = 0;
$mudsdown = 0;
$mudcount = 0;
foreach $mudid (sort { $muds{$a}{name} cmp $muds{$b}{name} } (keys(%muds)))
{
  $mudcount++;

  if ( $muds{$mudid}{status} eq "Up" )
  {
    $mudsup++;
  } else {
    $mudsdown++;
  }

  # produce the HTML output
  $mudname     = $muds{$mudid}{name};
  $mudurl      = $muds{$mudid}{url};
  $mudaddr     = $muds{$mudid}{addr};
  $mudcurrip   = $muds{$mudid}{currip};
  $mudport     = $muds{$mudid}{port};
  $mudlastconn = $muds{$mudid}{lastconn};
  $mudstatus   = $muds{$mudid}{status};
  $mudcomment  = $muds{$mudid}{comment};
  
  # Alternate the colours for each line in the table
  if ( $mudcount % 2 )
  {
    $HTMLcol = "colour1";
  } else {
    $HTMLcol = "colour2";
  }
  $mudinfo = "  <tr class=\"$HTMLcol\">\n  ";

  # Put the name of the mud in, and if we've got a url make it a link to it
  if ( $muds{$mudid}{url} ne "" )
  {
    $mudinfo = "$mudinfo<td><a href=\"$muds{$mudid}{url}\" title=\"View the website of $muds{$mudid}{name}\">$muds{$mudid}{name}</a></td>";
  } else {
    $mudinfo = "$mudinfo<td>$muds{$mudid}{name}</td>";
  }

  # The mud's address
  if ( $mudaddr eq "" )
  {
    $mudaddr = "nbsp;";
  }
  $mudinfo = "$mudinfo<td>$mudaddr</td>";

  # The mud's ip address
  $mudinfo = "$mudinfo<td>$mudcurrip</td>";

  # The mud's port
  $mudinfo = "$mudinfo<td align=\"center\"><a href=\"telnet://$mudaddr:$mudport\" title=\"Connect to $muds{$mudid}{name}\">$mudport</a></td>";

  # The mud's status
  $mudinfo = "$mudinfo<td align=\"center\">$mudstatus</td>";

  # Comments for the mud (if any)
  if ( $muds{$mudid}{status} eq "Down" && $muds{$mudid}{lastconn} ne "" )
  {
    # if the mud's down put the time of the last connection down
    $mudcomment = "Last up: " . localtime(timegm(split('\|',$muds{$mudid}{lastconn})));
  }
  if ( $mudcomment eq "" )
  {
    $mudcomment = "&nbsp;";
  }
  $mudinfo = "$mudinfo<td>$mudcomment</td>";

  # End of the record
  $mudinfo = "$mudinfo\n  </tr>\n";

  # Write the line to the file
  print TMP_HTML_FILE "$mudinfo";


  # produce the INFO output
  $mudname     = $muds{$mudid}{name};
  $mudurl      = $muds{$mudid}{url};
  $mudaddr     = $muds{$mudid}{addr};
  $mudcurrip   = $muds{$mudid}{currip};
  $mudport     = $muds{$mudid}{port};
  $mudlastconn = $muds{$mudid}{lastconn};
  $mudstatus   = $muds{$mudid}{status};
  $mudcomment  = $muds{$mudid}{comment};

  # Alternate the colours for each line in the table
  if ( $mudcount % 2 )
  {
    $INFOcol = $mudlist_config{'INFO_COL_LINE1'};
  } else {
    $INFOcol = $mudlist_config{'INFO_COL_LINE2'};
  }

  $mudinfo = "  <tr class=\"$HTMLcol\">\n  ";
  if ( $muds{$mudid}{status} eq "Up" )
  {
    $mudstatus = "$mudlist_config{'INFO_COL_UP'}Up";
  } else {
    $mudstatus = "$mudlist_config{'INFO_COL_DOWN'}Down";
  }

  $mudinfo = "";
  printf TMP_INFO_FILE "%s%-20s %-33s %-15s %-4s %s\n",
          $INFOcol,$mudname,$mudaddr,$mudcurrip,$mudport,$mudstatus;
  #print TMP_INFO_FILE $mudinfo;
}

# Get current time
$currtime = localtime(time);

# Print file footers for the HTML file and the info file
print TMP_HTML_FILE <<EOF;
</table>
<br />
<table>
  <tr>
    <td>Number of MUDS currently up:</td><td>$mudsup</td>
  </tr>
  <tr>
    <td>Number of MUDS currently down:</td><td>$mudsdown</td>
  </tr>
  <tr>
    <td>Total Number of MUDS in list:</td><td>$mudcount</td>
  </tr>
</table>
<br />
<p class="lastupdate">
Last updated: $currtime
</p>
<p class="serveradmin">
$mudlist_config{'HTML_TXT_CHANGES'} <a href="mailto:$mudlist_config{'SERVER_ADMIN'}" title="Send an email to $mudlist_config{'SERVER_ADMIN'}">$mudlist_config{'SERVER_ADMIN'}</a>
</p>
</center>
</body>
</html>
EOF

print TMP_INFO_FILE <<EOF;

                        $mudlist_config{'INFO_COL_MUDS_UP_TXT'}Number of MUDS currently up:   $mudlist_config{'INFO_COL_MUDS_UP_NUM'}$mudsup
                        $mudlist_config{'INFO_COL_MUDS_DOWN_TXT'}Number of MUDS currently down: $mudlist_config{'INFO_COL_MUDS_DOWN_NUM'}$mudsdown
                        $mudlist_config{'INFO_COL_TOTAL_MUDS_TXT'}Total number of muds in list:  $mudlist_config{'INFO_COL_TOTAL_MUDS_NUM'}$mudcount

                     $mudlist_config{'INFO_COL_UPDATED_TXT'}Last updated: $mudlist_config{'INFO_COL_UPDATED_DTE'}$currtime


EOF

# Where the web version of the mudlist is
$str = &centre_spaces($mudlist_config{'INFO_TXT_WEB_VER'});
print TMP_INFO_FILE "${str}$mudlist_config{'INFO_COL_WEB_VER'}$mudlist_config{'INFO_TXT_WEB_VER'}\n\n";
$str = &centre_spaces($mudlist_config{'HTML_FILE_URL'});
print TMP_INFO_FILE "${str}$mudlist_config{'INFO_COL_URL'}$mudlist_config{'HTML_FILE_URL'}\n\n";

# Who to contact for changes
$str = &centre_spaces($mudlist_config{'INFO_TXT_CHANGES'} . " " . $mudlist_config{'SERVER_ADMIN'});
print TMP_INFO_FILE "\n${str}$mudlist_config{'INFO_COL_CHANGES'}$mudlist_config{'INFO_TXT_CHANGES'} ";
print TMP_INFO_FILE "$mudlist_config{'INFO_COL_SERVER_ADMIN'}$mudlist_config{'SERVER_ADMIN'}\n\n\n";


# All done with the output files for HTML and INFO so close them
close(TMP_HTML_FILE);
close(TMP_INFO_FILE);

# move the temporary files to the live ones
#print "DEBUG: renaming $mudlist_config{'TMP_HTML_FILE'} to $mudlist_config{'HTML_FILE'}\n";
rename($mudlist_config{'TMP_HTML_FILE'},$mudlist_config{'HTML_FILE'});
#print "DEBUG: renaming $mudlist_config{'TMP_INFO_FILE'} to $mudlist_config{'INFO_FILE'}\n";
rename($mudlist_config{'TMP_INFO_FILE'},$mudlist_config{'INFO_FILE'});


#------------------------------------------------------------------------------
# small function to get number of spaces to centre a string assuming 80 columns
#------------------------------------------------------------------------------
sub centre_spaces()
{
  my $inpstr = $_[0];
  my $idx;
  my $spaces = "";
  my $no_of_spaces;

  $no_of_spaces = int((80-length($inpstr))/2);
  for ( $idx = $no_of_spaces ; $idx >= 0 ; $idx-- )
  {
    $spaces = "$spaces ";
  }

  return $spaces;
}


#******************************************************************************
#*** End of file: mudlist_client.pl                                        ****
#******************************************************************************
